@@ -273,12 +273,16 @@ void mainwindow::readSettings()
 
 	int sizee = settings.beginReadArray("OpenDBHistory");
 	QStringList dbhist;
+	QString ftoopen = checkForFileToOpen();
+	if (ftoopen.isEmpty() == false) dbhist << ftoopen;
 	for (int i=0; i < sizee; i++)
 	{
 		settings.setArrayIndex(i);
 		dbhist << settings.value("db").toString();
 	}
 	settings.endArray();
+	dbhist.removeDuplicates();
+	if (dbhist.count() > 7) dbhist.removeLast();
 	if (dbhist.isEmpty() == false) ui->comboBoxDB->addItems(dbhist);
 
 	int sizef = settings.beginReadArray("FilterHistory");
@@ -303,7 +307,8 @@ void mainwindow::readSettings()
 	ui->checkBoxSymbolOnly->setChecked(settings.value("SymbolOnly", false).toBool());
 	ui->checkBoxFilter->setChecked(settings.value("FilterCheckBox", false).toBool());
 	ui->comboBoxQueryType->setCurrentIndex(settings.value("QueryType", 0).toInt());
-	ui->comboBoxDB->setCurrentIndex(settings.value("LastOpenDB", ui->comboBoxDB->currentIndex()).toInt());
+	if (ftoopen.isEmpty() == false) ui->comboBoxDB->setCurrentIndex(0);
+	else ui->comboBoxDB->setCurrentIndex(settings.value("LastOpenDB", ui->comboBoxDB->currentIndex()).toInt());
 	m_currentLanguage = settings.value("Language", QString("English")).toString();
 	retranslateUi();
 	m_fileviewer->m_externalEditorPath =
@@ -342,6 +347,47 @@ void mainwindow::readSettings()
 
 }
 
+QString mainwindow::checkForFileToOpen(void)
+{
+	QStringList arg = m_app->arguments();
+	QString fn;
+	if (arg.size() <= 1) return fn;
+	if (arg.size() >= 3)
+	{
+		printHelpAndExit("ERROR: More than 1 argument is not recognized.");
+		return fn;
+	}
+	if ((arg[1] == "--help")||
+		(arg[1] == "-h")||
+		(arg[1] == "-?")||
+		(arg[1] == "/?"))
+	{
+		printHelpAndExit("");
+		return fn;
+	}
+	QFileInfo qfi(arg[1]);
+	if ((qfi.exists() == false) || (qfi.isFile() == false))
+	{
+		tempbuf buf(3000);
+		sprintf(buf.get(), "ERROR: File \"%s\" does not exist!", arg[1].toStdString().c_str());
+		printHelpAndExit(buf.get());
+		return fn;
+	}
+	fn = qfi.canonicalFilePath();
+	return fn;
+}
+
+void mainwindow::printHelpAndExit(QString str)
+{
+		printf("codequery [path_to_codequery_database_file_to_open]\n");
+		printf("The argument is optional.\n");
+		if (str.isEmpty() == false) printf("%s\n", str.toStdString().c_str());
+		printf("\n");
+		m_listhandler->prepareToExit();
+		m_app->quit();
+		exit(str.isEmpty() ? 0 : 1);
+}
+
 
 void mainwindow::resizeEvent(QResizeEvent* event)
 {
