@@ -64,6 +64,8 @@ QString m_currentLanguage;
 QTranslator m_translator;
 
 void init(void);
+QString checkForFileToOpen(void);
+void printHelpAndExit(QString str);
 
 protected:
 virtual void resizeEvent(QResizeEvent* event);
