@@ -15,7 +15,7 @@
 #include "graphdialog.h"
 #include "searchhandler.h"
 
-#ifdef USE_QT5
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 #include <QtConcurrent/QtConcurrent>
 #endif
 
