@@ -136,6 +136,7 @@ class LexerABL : public DefaultLexer {
    OptionSetABL osABL;
 public:
    LexerABL() :
+      DefaultLexer("abl", SCLEX_PROGRESS),
       setWord(CharacterSet::setAlphaNum, "_", 0x80, true),
       setNegationOp(CharacterSet::setNone, "!"),
       setArithmethicOp(CharacterSet::setNone, "+-/*%"),
@@ -148,7 +149,7 @@ public:
       delete this;
    }
    int SCI_METHOD Version() const override {
-      return lvOriginal;
+      return lvIdentity;
    }
    const char * SCI_METHOD PropertyNames() override {
       return osABL.PropertyNames();
@@ -160,6 +161,9 @@ public:
       return osABL.DescribeProperty(name);
    }
    Sci_Position SCI_METHOD PropertySet(const char *key, const char *val) override ;
+   const char * SCI_METHOD PropertyGet(const char *key) override {
+	   return osABL.PropertyGet(key);
+   }
 
    const char * SCI_METHOD DescribeWordListSets() override {
       return osABL.DescribeWordListSets();
